-------------------------------------------------------------------------------
-- axi_apb_bridge.vhd - entity/architecture pair
-------------------------------------------------------------------------------
--
-- ****************************************************************************
-- DISCLAIMER OF LIABILITY
--
-- This file contains proprietary and confidential information of
-- Xilinx, Inc. ("Xilinx"), that is distributed under a license
-- from Xilinx, and may be used, copied and/or disclosed only
-- pursuant to the terms of a valid license agreement with Xilinx.
--
-- XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION
-- ("MATERIALS") "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
-- EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING WITHOUT
-- LIMITATION, ANY WARRANTY WITH RESPECT TO NONINFRINGEMENT,
-- MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE. Xilinx
-- does not warrant that functions included in the Materials will
-- meet the requirements of Licensee, or that the operation of the
-- Materials will be uninterrupted or error-free, or that defects
-- in the Materials will be corrected. Furthermore, Xilinx does
-- not warrant or make any representations regarding use, or the
-- results of the use, of the Materials in terms of correctness,
-- accuracy, reliability or otherwise.
--
-- Xilinx products are not designed or intended to be fail-safe,
-- or for use in any application requiring fail-safe performance,
-- such as life-support or safety devices or systems, Class III
-- medical devices, nuclear facilities, applications related to
-- the deployment of airbags, or any other applications that could
-- lead to death, personal injury or severe property or
-- environmental damage (individually and collectively, "critical
-- applications"). Customer assumes the sole risk and liability
-- of any use of Xilinx products in critical applications,
-- subject only to applicable laws and regulations governing
-- limitations on product liability.
--
-- Copyright 2009-2012 Xilinx, Inc.
-- All rights reserved.
--
-- This disclaimer and copyright notice must be retained as part
-- of this file at all times.
-- ****************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        axi_apb_bridge.vhd
-- Version:         v1.01a
-- Description:     The AXI to APB Bridge module translates AXI
--                  transactions into APB transactions. It functions as a
--                  AXI slave on the AXI port and an APB master on
--                  the APB interface.
--
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:
--           -- axi_apb_bridge.vhd
--              -- psel_decoder.vhd
--              -- multiplexor.vhd
--              -- axilite_sif.vhd
--              -- apb_mif.vhd
--
-------------------------------------------------------------------------------
-- Author:      USM
-- History:
--   USM      07/30/2010   Initial version
-- ^^^^^^^
--   NLR      01/05/2012   Added the multiple slave support in RTL
-- ~~~~~~~
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library axi_apb_bridge_v1_01_a;

-------------------------------------------------------------------------------
-- Port Declaration
-------------------------------------------------------------------------------
--
-- Definition of Generics
--
-- System Parameters
--
--  C_FAMILY                 -- FPGA Family for which the axi_apb_bridge is
--                           -- targeted
--  C_INSTANCE               -- Instance name of the axi_apb_bridge in the
--                           -- system
-- AXI Parameters
--
--  C_S_AXI_ADDR_WIDTH       -- Width of the AXI address bus (in bits)
--                              fixed to 32
--  C_S_AXI_DATA_WIDTH       -- Width of the AXI data bus (in bits)
--                              fixed to 32
--  C_BASEADDR               -- AXI Low address for address range 1
--  C_HIGHADDR               -- AXI high address for address range 1
--  C_S_AXI_RNG2_BASEADDR    -- AXI low address for address range 2    
--  C_S_AXI_RNG2_HIGHADDR    -- AXI high address for address range 2
--  C_S_AXI_RNG3_BASEADDR    -- AXI low address for address range 3 
--  C_S_AXI_RNG3_HIGHADDR    -- AXI high address for address range 3 
--  C_S_AXI_RNG4_BASEADDR    -- AXI low address for address range 4 
--  C_S_AXI_RNG4_HIGHADDR    -- AXI high address for address range 4 
--  C_S_AXI_RNG5_BASEADDR    -- AXI low address for address range 5 
--  C_S_AXI_RNG5_HIGHADDR    -- AXI high address for address range 5 
--  C_S_AXI_RNG6_BASEADDR    -- AXI low address for address range 6 
--  C_S_AXI_RNG6_HIGHADDR    -- AXI high address for address range 6 
--  C_S_AXI_RNG7_BASEADDR    -- AXI low address for address range 7  
--  C_S_AXI_RNG7_HIGHADDR    -- AXI high address for address range 7 
--  C_S_AXI_RNG8_BASEADDR    -- AXI low address for address range 8 
--  C_S_AXI_RNG8_HIGHADDR    -- AXI high address for address range 8 
--  C_S_AXI_RNG9_BASEADDR    -- AXI low address for address range 9 
--  C_S_AXI_RNG9_HIGHADDR    -- AXI high address for address range 9 
--  C_S_AXI_RNG10_BASEADDR   -- AXI low address for address range 10 
--  C_S_AXI_RNG10_HIGHADDR   -- AXI high address for address range 10 
--  C_S_AXI_RNG11_BASEADDR   -- AXI low address for address range 11 
--  C_S_AXI_RNG11_HIGHADDR   -- AXI high address for address range 11 
--  C_S_AXI_RNG12_BASEADDR   -- AXI low address for address range 12
--  C_S_AXI_RNG12_HIGHADDR   -- AXI high address for address range 12
--  C_S_AXI_RNG13_BASEADDR   -- AXI low address for address range 13 
--  C_S_AXI_RNG13_HIGHADDR   -- AXI high address for address range 13 
--  C_S_AXI_RNG14_BASEADDR   -- AXI low address for address range 14 
--  C_S_AXI_RNG14_HIGHADDR   -- AXI high address for address range 14 
--  C_S_AXI_RNG15_BASEADDR   -- AXI low address for address range 15 
--  C_S_AXI_RNG15_HIGHADDR   -- AXI high address for address range 15 
--  C_S_AXI_RNG16_BASEADDR   -- AXI low address for address range 16 
--  C_S_AXI_RNG16_HIGHADDR   -- AXI high address for address range 16 
--
-- APB Parameters
--
--  C_M_APB_ADDR_WIDTH       -- Width of the APB address bus (in bits)
--                              fixed to 32
--  C_M_APB_DATA_WIDTH       -- Width of the APB data bus (in bits)
--                              fixed to 32
--  C_APB_NUM_SLAVES         -- The number of APB slaves
--  C_M_APB_PROTOCOL         -- The type of APB interface APB3/APB4
--
-- Core Parameters
--
--  C_DPHASE_TIMEOUT         -- Data phase time out value
--
-- Definition of Ports
--
-- System signals
--
--  S_AXI_ACLK               -- AXI Clock
--  S_AXI_ARESETN            -- AXI Reset Signal - active low
--
-- AXI Write address channel signals
--  S_AXI_AWADDR             -- Write address bus - The write address bus gives
--                              the address of the first transfer in a write
--                              burst transaction - fixed to 32
--  S_AXI_AWPROT             -- Protection type - This signal indicates the
--                              normal, privileged, or secure protection level
--                              of the transaction and whether the transaction
--                              is a data access or an instruction access
--  S_AXI_AWVALID            -- Write address valid - This signal indicates
--                              that valid write address & control information
--                              are available
--  S_AXI_AWREADY            -- Write address ready - This signal indicates
--                              that the slave is ready to accept an address
--                              and associated control signals
--
-- AXI Write data channel signals
--
--  S_AXI_WDATA              -- Write data bus - fixed to 32
--  S_AXI_WSTRB              -- Write strobes - These signals indicates which
--                              byte lanes to update in memory
--  S_AXI_WVALID             -- Write valid - This signal indicates that valid
--                              write data and strobes are available
--  S_AXI_WREADY             -- Write ready - This signal indicates that the
--                              slave can accept the write data
-- AXI Write response channel signals
--
--  S_AXI_BRESP              -- Write response - This signal indicates the
--                              status of the write transaction
--  S_AXI_BVALID             -- Write response valid - This signal indicates
--                              that a valid write response is available
--  S_AXI_BREADY             -- Response ready - This signal indicates that
--                              the master can accept the response information
--
-- AXI Read address channel signals
--
--  S_AXI_ARADDR             -- Read address - The read address bus gives the
--                              initial address of a read burst transaction
--  S_AXI_ARPROT             -- Protection type - This signal provides
--                              protection unit information for the transaction
--  S_AXI_ARVALID            -- Read address valid - This signal indicates,
--                              when HIGH, that the read address and control
--                              information is valid and will remain stable
--                              until the address acknowledge signal,ARREADY,
--                              is high.
--  S_AXI_ARREADY            -- Read address ready - This signal indicates
--                              that the slave is ready to accept an address
--                              and associated control signals:
--
-- AXI Read data channel signals
--
--  S_AXI_RDATA              -- Read data bus - fixed to 32
--  S_AXI_RRESP              -- Read response - This signal indicates the
--                              status of the read transfer
--  S_AXI_RVALID             -- Read valid - This signal indicates that the
--                              required read data is available and the read
--                              transfer can complete
--  S_AXI_RREADY             -- Read ready - This signal indicates that the
--                              master can accept the read data and response
--                              information
-- APB signals
--  M_APB_PCLK               -- APB Clock
--  M_APB_PRESETN            -- APB Reset Signal - active low
--  M_APB_PADDR              -- APB address bus
--  M_APB_PSEL               -- Slave select signal
--  M_APB_PENABLE            -- Enable signal indicates that the second and
--                              sub-sequent cycles of an APB transfer
--  M_APB_PWRITE             -- Direction indicates an APB write access when
--                              high and an APB read access when low
--  M_APB_PWDATA             -- APB write data
--  M_APB_PREADY             -- Ready, the APB slave uses this signal to
--                              extend an APB transfer
--  M_APB_PRDATA1            -- APB read data driven by slave 1
--  M_APB_PRDATA2            -- APB read data driven by slave 2
--  M_APB_PRDATA3            -- APB read data driven by slave 3
--  M_APB_PRDATA4            -- APB read data driven by slave 4
--  M_APB_PRDATA5            -- APB read data driven by slave 5
--  M_APB_PRDATA6            -- APB read data driven by slave 6
--  M_APB_PRDATA7            -- APB read data driven by slave 7
--  M_APB_PRDATA8            -- APB read data driven by slave 8
--  M_APB_PRDATA9            -- APB read data driven by slave 9
--  M_APB_PRDATA10           -- APB read data driven by slave 10
--  M_APB_PRDATA11           -- APB read data driven by slave 11
--  M_APB_PRDATA12           -- APB read data driven by slave 12
--  M_APB_PRDATA13           -- APB read data driven by slave 13
--  M_APB_PRDATA14           -- APB read data driven by slave 14
--  M_APB_PRDATA15           -- APB read data driven by slave 15
--  M_APB_PRDATA16           -- APB read data driven by slave 16
--  M_APB_PSLVERR            -- This signal indicates transfer failure
--  M_APB_PPROT              -- This signal indicates the normal,
--                              privileged, or secure protection level of the
--                              transaction and whether the transaction is a
--                              data access or an instruction access. Driven
--                              when APB4 is selected.
--  M_APB_PSTRB              -- Write strobes. This signal indicates which
--                              byte lanes to update during a write transfer.
--                              Write strobes must not be active during a
--                              read transfer.Driven when APB4 is selected.
-------------------------------------------------------------------------------
-- Generics & Signals Description
-------------------------------------------------------------------------------

entity axi_apb_bridge is
  generic (
    C_FAMILY              : string                    := "virtex6";
    C_INSTANCE            : string                    := "axi_apb_bridge_inst";

    C_S_AXI_ADDR_WIDTH    : integer range 32 to 32    := 32;
    C_S_AXI_DATA_WIDTH    : integer range 32 to 32    := 32;

    C_M_APB_ADDR_WIDTH    : integer range 32 to 32    := 32;
    C_M_APB_DATA_WIDTH    : integer range 32 to 32    := 32;
    C_APB_NUM_SLAVES      : integer range 1 to 16     := 4;
    C_M_APB_PROTOCOL      : string                    := "apb3";

    C_BASEADDR            : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_HIGHADDR            : std_logic_vector(0 to 31) := X"00000000";
    C_S_AXI_RNG2_BASEADDR : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_S_AXI_RNG2_HIGHADDR : std_logic_vector(0 to 31) := X"00000000";
    C_S_AXI_RNG3_BASEADDR : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_S_AXI_RNG3_HIGHADDR : std_logic_vector(0 to 31) := X"00000000";
    C_S_AXI_RNG4_BASEADDR : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_S_AXI_RNG4_HIGHADDR : std_logic_vector(0 to 31) := X"00000000";
    C_S_AXI_RNG5_BASEADDR : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_S_AXI_RNG5_HIGHADDR : std_logic_vector(0 to 31) := X"00000000";
    C_S_AXI_RNG6_BASEADDR : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_S_AXI_RNG6_HIGHADDR : std_logic_vector(0 to 31) := X"00000000";
    C_S_AXI_RNG7_BASEADDR : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_S_AXI_RNG7_HIGHADDR : std_logic_vector(0 to 31) := X"00000000";
    C_S_AXI_RNG8_BASEADDR : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_S_AXI_RNG8_HIGHADDR : std_logic_vector(0 to 31) := X"00000000";
    C_S_AXI_RNG9_BASEADDR : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_S_AXI_RNG9_HIGHADDR : std_logic_vector(0 to 31) := X"00000000";
    C_S_AXI_RNG10_BASEADDR : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_S_AXI_RNG10_HIGHADDR : std_logic_vector(0 to 31) := X"00000000";
    C_S_AXI_RNG11_BASEADDR : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_S_AXI_RNG11_HIGHADDR : std_logic_vector(0 to 31) := X"00000000";
    C_S_AXI_RNG12_BASEADDR : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_S_AXI_RNG12_HIGHADDR : std_logic_vector(0 to 31) := X"00000000";
    C_S_AXI_RNG13_BASEADDR : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_S_AXI_RNG13_HIGHADDR : std_logic_vector(0 to 31) := X"00000000";
    C_S_AXI_RNG14_BASEADDR : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_S_AXI_RNG14_HIGHADDR : std_logic_vector(0 to 31) := X"00000000";
    C_S_AXI_RNG15_BASEADDR : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_S_AXI_RNG15_HIGHADDR : std_logic_vector(0 to 31) := X"00000000";
    C_S_AXI_RNG16_BASEADDR : std_logic_vector(0 to 31) := X"FFFFFFFF";
    C_S_AXI_RNG16_HIGHADDR : std_logic_vector(0 to 31) := X"00000000";
    C_DPHASE_TIMEOUT      : integer range 0 to 256    := 0
    );
  port (
  -- AXI signals
    S_AXI_ACLK         : in  std_logic;
    S_AXI_ARESETN      : in  std_logic;

    S_AXI_AWADDR       : in  std_logic_vector(31 downto 0);
    S_AXI_AWPROT       : in  std_logic_vector(2 downto 0);
    S_AXI_AWVALID      : in  std_logic;
    S_AXI_AWREADY      : out std_logic;
    S_AXI_WDATA        : in  std_logic_vector(31 downto 0);
    S_AXI_WSTRB        : in  std_logic_vector
                             (3 downto 0);
    S_AXI_WVALID       : in  std_logic;
    S_AXI_WREADY       : out std_logic;
    S_AXI_BRESP        : out std_logic_vector(1 downto 0);
    S_AXI_BVALID       : out std_logic;
    S_AXI_BREADY       : in  std_logic;

    S_AXI_ARADDR       : in  std_logic_vector(31 downto 0);
    S_AXI_ARPROT       : in  std_logic_vector(2 downto 0);
    S_AXI_ARVALID      : in  std_logic;
    S_AXI_ARREADY      : out std_logic;
    S_AXI_RDATA        : out std_logic_vector(31 downto 0);
    S_AXI_RRESP        : out std_logic_vector(1 downto 0);
    S_AXI_RVALID       : out std_logic;
    S_AXI_RREADY       : in  std_logic;

-- APB signals
    M_APB_PCLK         : out  std_logic;
    M_APB_PRESETN      : out  std_logic;
    M_APB_PADDR        : out std_logic_vector(31 downto 0);
    M_APB_PSEL         : out std_logic_vector(C_APB_NUM_SLAVES-1 downto 0);
    M_APB_PENABLE      : out std_logic;
    M_APB_PWRITE       : out std_logic;
    M_APB_PWDATA       : out std_logic_vector(31 downto 0);
    M_APB_PREADY       : in  std_logic_vector(C_APB_NUM_SLAVES-1 downto 0);
    M_APB_PRDATA       : in  std_logic_vector(31 downto 0);
    M_APB_PRDATA2      : in  std_logic_vector(31 downto 0);
    M_APB_PRDATA3      : in  std_logic_vector(31 downto 0);
    M_APB_PRDATA4      : in  std_logic_vector(31 downto 0);
    M_APB_PRDATA5      : in  std_logic_vector(31 downto 0);
    M_APB_PRDATA6      : in  std_logic_vector(31 downto 0);
    M_APB_PRDATA7      : in  std_logic_vector(31 downto 0);
    M_APB_PRDATA8      : in  std_logic_vector(31 downto 0);
    M_APB_PRDATA9      : in  std_logic_vector(31 downto 0);
    M_APB_PRDATA10     : in  std_logic_vector(31 downto 0);
    M_APB_PRDATA11     : in  std_logic_vector(31 downto 0);
    M_APB_PRDATA12     : in  std_logic_vector(31 downto 0);
    M_APB_PRDATA13     : in  std_logic_vector(31 downto 0);
    M_APB_PRDATA14     : in  std_logic_vector(31 downto 0);
    M_APB_PRDATA15     : in  std_logic_vector(31 downto 0);
    M_APB_PRDATA16     : in  std_logic_vector(31 downto 0);
    M_APB_PSLVERR      : in  std_logic_vector(C_APB_NUM_SLAVES-1 downto 0);
    M_APB_PPROT        : out std_logic_vector(2 downto 0);
    M_APB_PSTRB        : out std_logic_vector
                             (3 downto 0)
    );

end entity axi_apb_bridge;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------

architecture RTL of axi_apb_bridge is

    constant C_CORE_GENERATION_INFO : string := C_INSTANCE & ",axi_apb_bridge,{"
      & "c_family="                    & C_FAMILY
      & ",c_instance="                 & C_INSTANCE
      & ",c_m_apb_protocol="           & C_M_APB_PROTOCOL
      & ",c_apb_num_slaves="           & integer'image(C_APB_NUM_SLAVES)
      & ",c_dphase_timeout="           & integer'image(C_DPHASE_TIMEOUT)
      & "}";

    attribute CORE_GENERATION_INFO : string;
    attribute CORE_GENERATION_INFO of RTL : architecture is C_CORE_GENERATION_INFO;

-------------------------------------------------------------------------------
 -- Signal declarations
-------------------------------------------------------------------------------

    signal Address        : std_logic_vector(C_S_AXI_ADDR_WIDTH-1 downto 0);
    signal sl_pselect     : std_logic_vector(C_APB_NUM_SLAVES-1 downto 0);
    signal apb_pslverr    : std_logic;
    signal apb_pready     : std_logic;
    signal apb_prdata     : std_logic_vector(C_M_APB_DATA_WIDTH-1 downto 0);
    signal apb_rd_request : std_logic;
    signal apb_wr_request : std_logic;
    signal rd_data        : std_logic_vector(C_M_APB_DATA_WIDTH-1 downto 0);
    signal slv_err_resp   : std_logic;
    signal PSEL_i         : std_logic;
    signal axi_awprot     : std_logic_vector(2 downto 0);
    signal dphase_timeout : std_logic;    
    
begin

-------------------------------------------------------------------------------
-- Begin architecture logic
-------------------------------------------------------------------------------
    
-------------------------------------------------------------------------------
-- APB clock and reset assignments
-------------------------------------------------------------------------------

    M_APB_PCLK <= S_AXI_ACLK;
    
    M_APB_PRESETN <= S_AXI_ARESETN;

-------------------------------------------------------------------------------
 -- Instantiate the address decoder as APB is shared
-------------------------------------------------------------------------------

        PSEL_DECODER_MODULE : entity axi_apb_bridge_v1_01_a.psel_decoder
        generic map
        (
         C_FAMILY                         => C_FAMILY,
         C_S_AXI_ADDR_WIDTH               => C_S_AXI_ADDR_WIDTH,
         C_APB_NUM_SLAVES                 => C_APB_NUM_SLAVES,
         C_S_AXI_RNG1_BASEADDR            => C_BASEADDR,
         C_S_AXI_RNG1_HIGHADDR            => C_HIGHADDR,
         C_S_AXI_RNG2_BASEADDR            => C_S_AXI_RNG2_BASEADDR,
         C_S_AXI_RNG2_HIGHADDR            => C_S_AXI_RNG2_HIGHADDR,
         C_S_AXI_RNG3_BASEADDR            => C_S_AXI_RNG3_BASEADDR,
         C_S_AXI_RNG3_HIGHADDR            => C_S_AXI_RNG3_HIGHADDR,
         C_S_AXI_RNG4_BASEADDR            => C_S_AXI_RNG4_BASEADDR,
         C_S_AXI_RNG4_HIGHADDR            => C_S_AXI_RNG4_HIGHADDR,
         C_S_AXI_RNG5_BASEADDR            => C_S_AXI_RNG5_BASEADDR,
         C_S_AXI_RNG5_HIGHADDR            => C_S_AXI_RNG5_HIGHADDR,
         C_S_AXI_RNG6_BASEADDR            => C_S_AXI_RNG6_BASEADDR,
         C_S_AXI_RNG6_HIGHADDR            => C_S_AXI_RNG6_HIGHADDR,
         C_S_AXI_RNG7_BASEADDR            => C_S_AXI_RNG7_BASEADDR,
         C_S_AXI_RNG7_HIGHADDR            => C_S_AXI_RNG7_HIGHADDR,
         C_S_AXI_RNG8_BASEADDR            => C_S_AXI_RNG8_BASEADDR,
         C_S_AXI_RNG8_HIGHADDR            => C_S_AXI_RNG8_HIGHADDR,
         C_S_AXI_RNG9_BASEADDR            => C_S_AXI_RNG9_BASEADDR,
         C_S_AXI_RNG9_HIGHADDR            => C_S_AXI_RNG9_HIGHADDR,
         C_S_AXI_RNG10_BASEADDR           => C_S_AXI_RNG10_BASEADDR,
         C_S_AXI_RNG10_HIGHADDR           => C_S_AXI_RNG10_HIGHADDR,
         C_S_AXI_RNG11_BASEADDR           => C_S_AXI_RNG11_BASEADDR,
         C_S_AXI_RNG11_HIGHADDR           => C_S_AXI_RNG11_HIGHADDR,
         C_S_AXI_RNG12_BASEADDR           => C_S_AXI_RNG12_BASEADDR,
         C_S_AXI_RNG12_HIGHADDR           => C_S_AXI_RNG12_HIGHADDR,
         C_S_AXI_RNG13_BASEADDR           => C_S_AXI_RNG13_BASEADDR,
         C_S_AXI_RNG13_HIGHADDR           => C_S_AXI_RNG13_HIGHADDR,
         C_S_AXI_RNG14_BASEADDR           => C_S_AXI_RNG14_BASEADDR,
         C_S_AXI_RNG14_HIGHADDR           => C_S_AXI_RNG14_HIGHADDR,
         C_S_AXI_RNG15_BASEADDR           => C_S_AXI_RNG15_BASEADDR,
         C_S_AXI_RNG15_HIGHADDR           => C_S_AXI_RNG15_HIGHADDR,
         C_S_AXI_RNG16_BASEADDR           => C_S_AXI_RNG16_BASEADDR,
         C_S_AXI_RNG16_HIGHADDR           => C_S_AXI_RNG16_HIGHADDR
        )
        port map
        (
         Address                          => address,
         addr_is_valid                    => '1',
         sl_pselect                       => sl_pselect
        );
-------------------------------------------------------------------------------
 -- Instantiate the Multiplexor as APB is shared
-------------------------------------------------------------------------------

        MULTIPLEXOR_MODULE : entity axi_apb_bridge_v1_01_a.multiplexor
        generic map
        (
         C_M_APB_DATA_WIDTH               => C_M_APB_DATA_WIDTH,
         C_APB_NUM_SLAVES                 => C_APB_NUM_SLAVES
        )
        port map
        (
         M_APB_PCLK                       => S_AXI_ACLK,
         M_APB_PRESETN                    => S_AXI_ARESETN,
         M_APB_PREADY                     => M_APB_PREADY,
         M_APB_PRDATA1                    => M_APB_PRDATA,
         M_APB_PRDATA2                    => M_APB_PRDATA2,
         M_APB_PRDATA3                    => M_APB_PRDATA3,
         M_APB_PRDATA4                    => M_APB_PRDATA4,
         M_APB_PRDATA5                    => M_APB_PRDATA5,
         M_APB_PRDATA6                    => M_APB_PRDATA6,
         M_APB_PRDATA7                    => M_APB_PRDATA7,
         M_APB_PRDATA8                    => M_APB_PRDATA8,
         M_APB_PRDATA9                    => M_APB_PRDATA9,
         M_APB_PRDATA10                   => M_APB_PRDATA10,
         M_APB_PRDATA11                   => M_APB_PRDATA11,
         M_APB_PRDATA12                   => M_APB_PRDATA12,
         M_APB_PRDATA13                   => M_APB_PRDATA13,
         M_APB_PRDATA14                   => M_APB_PRDATA14,
         M_APB_PRDATA15                   => M_APB_PRDATA15,
         M_APB_PRDATA16                   => M_APB_PRDATA16,
         M_APB_PSLVERR                    => M_APB_PSLVERR,
         M_APB_PSEL                       => M_APB_PSEL,
         PSEL_i                           => PSEL_i,
         apb_pslverr                      => apb_pslverr,
         apb_pready                       => apb_pready,
         apb_prdata                       => apb_prdata,
         sl_pselect                       => sl_pselect
        );

-------------------------------------------------------------------------------
 -- Instantiate the AXI Lite Slave Interface module
-------------------------------------------------------------------------------

        AXILITE_SLAVE_IF_MODULE : entity axi_apb_bridge_v1_01_a.axilite_sif
        generic map
        (
         C_FAMILY                         => C_FAMILY,
         C_S_AXI_ADDR_WIDTH               => C_S_AXI_ADDR_WIDTH,
         C_S_AXI_DATA_WIDTH               => C_S_AXI_DATA_WIDTH,
         C_DPHASE_TIMEOUT                 => C_DPHASE_TIMEOUT,
         C_M_APB_PROTOCOL                 => C_M_APB_PROTOCOL
        )
        port map
        (
         S_AXI_ACLK                       => S_AXI_ACLK,
         S_AXI_ARESETN                    => S_AXI_ARESETN,

         S_AXI_AWADDR                     => S_AXI_AWADDR,
         S_AXI_AWPROT                      => S_AXI_AWPROT,
         S_AXI_AWVALID                    => S_AXI_AWVALID,
         S_AXI_AWREADY                    => S_AXI_AWREADY,
         S_AXI_WVALID                     => S_AXI_WVALID,
         S_AXI_WREADY                     => S_AXI_WREADY,
         S_AXI_BRESP                      => S_AXI_BRESP,
         S_AXI_BVALID                     => S_AXI_BVALID,
         S_AXI_BREADY                     => S_AXI_BREADY,

         S_AXI_ARADDR                     => S_AXI_ARADDR,
         S_AXI_ARVALID                    => S_AXI_ARVALID,
         S_AXI_ARREADY                    => S_AXI_ARREADY,
         S_AXI_RDATA                      => S_AXI_RDATA,
         S_AXI_RRESP                      => S_AXI_RRESP,
         S_AXI_RVALID                     => S_AXI_RVALID,
         S_AXI_RREADY                     => S_AXI_RREADY,
         axi_awprot                       => axi_awprot,
         address                          => address,
         apb_rd_request                   => apb_rd_request,
         apb_wr_request                   => apb_wr_request,
         dphase_timeout                   => dphase_timeout,
         apb_pready                       => apb_pready,
         slv_err_resp                     => slv_err_resp,
         rd_data                          => rd_data
        );

-------------------------------------------------------------------------------
 -- Instantiate the APB Master Interface module
-------------------------------------------------------------------------------

        APB_MASTER_IF_MODULE : entity axi_apb_bridge_v1_01_a.apb_mif
        generic map
        (
         C_M_APB_ADDR_WIDTH               => C_M_APB_ADDR_WIDTH,
         C_M_APB_DATA_WIDTH               => C_M_APB_DATA_WIDTH,
         C_S_AXI_DATA_WIDTH               => C_S_AXI_DATA_WIDTH,
         C_APB_NUM_SLAVES                 => C_APB_NUM_SLAVES,
         C_M_APB_PROTOCOL                 => C_M_APB_PROTOCOL
        )
        port map
        (
         M_APB_PCLK                       => S_AXI_ACLK,
         M_APB_PRESETN                    => S_AXI_ARESETN,

         M_APB_PADDR                      => M_APB_PADDR,
         M_APB_PENABLE                    => M_APB_PENABLE,
         M_APB_PWRITE                     => M_APB_PWRITE,
         M_APB_PWDATA                     => M_APB_PWDATA,
         M_APB_PSTRB                      => M_APB_PSTRB,
         M_APB_PPROT                      => M_APB_PPROT,
         apb_pslverr                      => apb_pslverr,
         apb_pready                       => apb_pready,
         apb_rd_request                   => apb_rd_request,
         apb_wr_request                   => apb_wr_request,
         dphase_timeout                   => dphase_timeout,
         apb_prdata                       => apb_prdata,
         rd_data                          => rd_data,
         slv_err_resp                     => slv_err_resp,
         PSEL_i                           => PSEL_i,
         address                          => address,
         axi_awprot                       => axi_awprot,
         S_AXI_WDATA                      => S_AXI_WDATA,
         S_AXI_WSTRB                      => S_AXI_WSTRB,
         S_AXI_ARADDR                     => S_AXI_ARADDR,
         S_AXI_ARPROT                     => S_AXI_ARPROT
        );

end architecture RTL;
